BYTE = BYTES = 1
KiB = BYTE << 10
MiB = KiB << 10
GiB = MiB << 10
TiB = GiB << 10

SECOND = SECONDS = 1
MINUTE = MINUTES = 60 * SECONDS
HOUR = HOURS = 60 * MINUTES
DAY = DAYS = 24 * HOURS  # usually

MILLISECOND = MILLISECONDS = SECOND / 1000
MICROSECOND = MICROSECONDS = MILLISECOND / 1000
